//.b 	
//	Copyright 2001 [MEAN].BOT
//	
//	File: MI_Sound.cpp
//	
//	[MEAN] Sound Module. 
//

	#include "MI_Sound.h"
	#include "MB_InterfaceL1.h"
	#include "MA_EngDLLCmd.h"
	#include "MG_Lib.h"

// define the global sound file objects.
	tclSndFileList vclSndFileList;

	tclSoundCmdList vclSoundCmdList_System( ctxSound_System );
	tclSoundCmdList vclSoundCmdList_Admin( ctxSound_Admin );
	tclSoundCmdList vclSoundCmdList_Public( ctxSound_Public );
	tclSoundCmdList vclSoundCmdList_BotRandom( ctxSound_BotRandom );
	tclSoundCmdList vclSoundCmdList_Damage( ctxSound_Damage );

// *** BEGIN
// *** tclSndFileList
// ***

	tclSndFileList::tclSndFileList (void)
	{
		vinNumSndFiles = 0;
		vboAllPrecached = false;

		for ( int vinIdx = 0; vinIdx < cinMaxSndFiles; vinIdx++ )
			vtx_arySndFile [vinIdx] = "";

	} // tclSndFileList::tclSndFileList

	int tclSndFileList::finNumSndFiles( void )
	{
		return vinNumSndFiles;
	}

	bool tclSndFileList::fboIdxIsValid( int pinIdx )
	{
		if ( vinNumSndFiles > cinMaxSndFiles || vinNumSndFiles < 0 )
		{ // something has gone wrong with the file count. As a result
		  // the index passed is not valid.
			return false;
		}

		if ( pinIdx < 0 || pinIdx >= vinNumSndFiles )
		{ // Index is out of range.
			return false;
		}
		
		return true;

	} // tclSndFileList::fboIdxIsValid

	int tclSndFileList::finFindSndFileIdx( string ptxSndFile )
	{
		if (vinNumSndFiles > cinMaxSndFiles || vinNumSndFiles < 0)
		{ // something has gone wrong with the file count. Do not
		  // perform search and indicate no match was found.
			return -1;
		}

		for (int vinIdx = 0; vinIdx < vinNumSndFiles; vinIdx++)
		{
			if (vtx_arySndFile [vinIdx] == ptxSndFile)
			{ 	
				// if we get here, a match was found.
				return vinIdx;	
			}
		}
		// if we get here, no match was found.
		return -1;

	} // tclSndFileList::finFindSndFileIdx

	int tclSndFileList::finAddSndFileName( string ptxSndFile )
	{
		if ( ptxSndFile.empty( ) )
		{ // nothing to add.
			return -1;
		}

		int vinIdx = finFindSndFileIdx (ptxSndFile);

		if (vinIdx >= 0)
		{ // File already in list, return index.
			return vinIdx;
		}

		if (vinNumSndFiles >= cinMaxSndFiles || vinNumSndFiles < 0)
		{ // List is full or something has gone wrong with the file 
		  // count. Do not add to the list.
			return -1;
		}

		vinIdx = vinNumSndFiles;
		vinNumSndFiles++;

		vtx_arySndFile [vinIdx] = ptxSndFile;
		return vinIdx;

	} // tclSndFileList::finAddSndFileName

	string tclSndFileList::ftxGetSndFileName (int pinIdx)
	{
		if (!fboIdxIsValid(pinIdx))
		{
			// invalid index, return empty string to indicate nothing
			// was found.
			return "";
		}

		return vtx_arySndFile [pinIdx];

	} // tclSndFileList::ftxGetSndFileName

	void tclSndFileList::fvoPrecacheSndFile (int pinIdx)
	{
		if ( !fboIdxIsValid( pinIdx ) )
		{ // Index is not valid.
			return;
		}

//		if ( vtx_arySndFile [pinIdx].vboEnabled )
		// only precache sounds that are enabled;
			finPrecacheSound( vtx_arySndFile [pinIdx] );
		
	} // tclSndFileList::fvoPrecacheSndFile

	void tclSndFileList::fvoPrecacheAllSndFiles (void)
	{
		for (int vinIdx = 0; vinIdx < cinMaxSndFiles; vinIdx++)
			fvoPrecacheSndFile( vinIdx );
		
		vboAllPrecached = true;

	} // tclSndFileList::fvoPrecacheAllSndFiles

// *** 
// *** tclSndFileList
// *** END.

// *** BEGIN.
// *** tclSoundCmd
// ***

	tclSoundCmd::tclSoundCmd(void)
	{
		vtxSndCmd = "";
		for (int vinIdx = 0 ; vinIdx < cinMaxRndSnd; vinIdx++)
		{
			vin_aryRndSnd [vinIdx] = -1;
		}
		
		vinNumRndSnd = 0;
		vboPrecached = false;
		vboEnabled = false;

	} // tclSoundCmd::tclSoundCmd

	void tclSoundCmd::fvoSetSndCmd( string ptxSndCmd )
	{
		vtxSndCmd = ptxSndCmd;
		vboEnabled = true;
	} // tclSoundCmd::fvoSetSndCmd
	
	string tclSoundCmd::ftxGetSndCmd( void )
	{
		return vtxSndCmd;

	} // tclSoundCmd::ftxGetSndCmd

	bool tclSoundCmd::fboAddRndSnd( string ptxSndFile )
	{
		if ( ptxSndFile.empty( ) )
		{ // nothing to add.
			return false;
		}

		if ( vinNumRndSnd >= cinMaxRndSnd || vinNumRndSnd < 0 )
		{ // List is full or something is wrong with counter.
			return false;
		}
		
		int vinSndIdx = vclSndFileList.finAddSndFileName( ptxSndFile );

		if ( vinSndIdx < 0 )
		{ // sound file did not get added to the list - do not proceed.
			return false;
		}

		// search for a previous entry to prevent duplicates.
		for ( int vinIdx = 0; vinIdx < vinNumRndSnd; vinIdx++ )
		{
			if ( vin_aryRndSnd[ vinIdx ] == vinSndIdx )
			{ // already in list - do not proceed but indicate entry was successful.
				return true;
			}
		}

		vin_aryRndSnd[ vinNumRndSnd ] = vinSndIdx;
		vinNumRndSnd++;
		return true;

	} // tclSoundCmd::fboAddRndSnd

	string tclSoundCmd::ftxGetRndSnd( void )
	{ // return a sound file name from random search.

		if ( vinNumRndSnd > cinMaxRndSnd || vinNumRndSnd <= 0 )
		{ // List is empty or something is wrong with counter.
			return "";
		}
		
		int vinRndIdx;
		string vtxSndFile;

		vinRndIdx = random( vinNumRndSnd );

		if ( vinRndIdx < 0 || vinRndIdx >= vinNumRndSnd )
		{ // invalid index from random generator. return empty string.
			return "";
		}

		vtxSndFile = vclSndFileList.ftxGetSndFileName( vin_aryRndSnd[ vinRndIdx ] );
		return vtxSndFile;
	} // tclSoundCmd::ftxGetRndSnd

	void tclSoundCmd::fvoPrecacheSndFiles( void )
	{
		for ( int vinIdx = 0; vinIdx < vinNumRndSnd; vinIdx++)
			vclSndFileList.fvoPrecacheSndFile( vin_aryRndSnd[ vinIdx ] );

		vboPrecached = true;
	} // tclSoundCmd::fvoPrecacheSndFiles

// *** 
// *** tclSoundCmd
// *** END.

// *** BEGIN.
// *** tclSoundCmdList
// ***
	
	tclSoundCmdList::tclSoundCmdList( string ptxSndList )
	{
		// vclSoundCmd is initialized by definition
		vinNumSndCmd = 0;
		vinCurSndCmdIdx = -1;
		vboSoundsAreEnabled = false;
		vboSoundsAreOn = false;
		vboAllPrecached = false;
		vtxSoundCmdListHeader = ptxSndList;

	} // tclSoundCmdList::tclSoundCmdList

	bool tclSoundCmdList::finCmdIdxIsValid( int pinCmdIdx )
	{
		if ( vinNumSndCmd < 0 || vinNumSndCmd > cinMaxSoundCmd )
		{ // Something is wrong with counter.
			return false;
		}
		
		if ( pinCmdIdx < 0 || pinCmdIdx >= vinNumSndCmd )
		{ // index out of range.
			return false;
		}
		return true;
	} // tclSoundCmdList::finCmdIdxIsValid

	int tclSoundCmdList::finFindSoundCmd( string ptxSoundCmd )
	{
		if ( vinNumSndCmd <= 0 || vinNumSndCmd > cinMaxSoundCmd )
		{ // list is empty or something is wrong with counter.
			return -1;
		}

		for ( int vinIdx = 0; vinIdx < vinNumSndCmd; vinIdx++ )
		{
			if ( vclSoundCmd[ vinIdx ].ftxGetSndCmd( ) == ptxSoundCmd )
				return vinIdx;
		}
		// if we get here command was not found or was disabled.
		return -1;

	}; // tclSoundCmdList::finFindSoundCmd

	int tclSoundCmdList::finAddSoundCmd( string ptxSoundCmd )
	{
		if ( vinNumSndCmd < 0 || vinNumSndCmd > cinMaxSoundCmd )
		{ // Something is wrong with counter.
			vclDebugTrace.fvoWriteEventInfo 
				( true, "ERROR: Cannot add sound command \"" + ptxSoundCmd + "\". Internal error." );
			vinCurSndCmdIdx = -1;
			return -1;
		}
		
		int vinIdx = finFindSoundCmd( ptxSoundCmd );
		
		if ( vinIdx >= 0 )
		{ // Command already exists. Return pointer to command.
			vinCurSndCmdIdx = vinIdx;
			return vinIdx;
		}
			
		if ( vinNumSndCmd == cinMaxSoundCmd )
		{ // List is full. Cannot add command.
			vclDebugTrace.fvoWriteEventInfo 
				( true, "WARNING: Cannot add sound command \"" + ptxSoundCmd + "\". The list is full." );
			vinCurSndCmdIdx = -1;
			return -1;
		}

		vinIdx = vinNumSndCmd;
		vinCurSndCmdIdx = vinIdx;
		vclSoundCmd[ vinIdx ].fvoSetSndCmd( ptxSoundCmd );
		vinNumSndCmd++;
		return vinIdx;

	}; // tclSoundCmdList::finAddSoundCmd

	bool tclSoundCmdList::fboAddSound( string ptxSndFile )
	{
		return fboAddSound( vinCurSndCmdIdx, ptxSndFile );
	}; // tclSoundCmdList::fboAddSound

	bool tclSoundCmdList::fboAddSound( int pinCmdIdx, string ptxSndFile )
	{
		if ( vinNumSndCmd < 0 || vinNumSndCmd > cinMaxSoundCmd )
		{ // Something is wrong with counter.
			vclDebugTrace.fvoWriteEventInfo 
				( true, "ERROR: Cannot add sound file \"" + ptxSndFile + "\". Internal error." );
			return false;
		}
		
		if ( pinCmdIdx < 0 || pinCmdIdx >= vinNumSndCmd )
		{ // index out of range.
			vclDebugTrace.fvoWriteEventInfo 
				( true, "WARNING: Cannot add sound file \"" + ptxSndFile + "\". The list is full." );
			return false;
		}
		
		return vclSoundCmd[ pinCmdIdx ].fboAddRndSnd( ptxSndFile );

	}; // tclSoundCmdList::fboAddSound

	bool tclSoundCmdList::fboAddSound(string ptxSoundCmd, string ptxSndFile)
	{
		int vinIdx = finFindSoundCmd(ptxSoundCmd);

		if ( vinIdx < 0 || vinIdx >= vinNumSndCmd )
		{ // sound command was not found.
			return false;
		}
		
		return vclSoundCmd[ vinIdx ].fboAddRndSnd( ptxSndFile );

	}; // tclSoundCmdList::fboAddSound

	string tclSoundCmdList::ftxGetSoundFile( int pinCmdIdx )
	{
		if ( !fboSoundsAreReady( ) )
			return "";

		if ( !finCmdIdxIsValid( pinCmdIdx ) )
		{ // Something is wrong with counter or index out of range.
			return "";
		}

		if ( !vclSoundCmd[ pinCmdIdx ].vboEnabled )
		{ // sound command is disabled;
			return "";
		}

		return  vclSoundCmd[ pinCmdIdx ].ftxGetRndSnd( );

	}; // tclSoundCmdList::ftxGetSoundFile

	string tclSoundCmdList::ftxGetSoundFile( string ptxSoundCmd )
	{
		if ( !fboSoundsAreReady( ) )
			return "";

		int vinIdx = finFindSoundCmd( ptxSoundCmd );

		if ( vinIdx < 0 )
			return "";

		if ( !vclSoundCmd[ vinIdx ].vboEnabled )
		{ // sound command is disabled;
			return "";
		}

		return  vclSoundCmd[ vinIdx ].ftxGetRndSnd( );

	}; // tclSoundCmdList::ftxGetSoundFile

	void tclSoundCmdList::fvoPrecacheSndFiles( void )
	{
		if ( !vboSoundsAreEnabled )
		{
			vboAllPrecached = false;
			return;
		}

		for ( int vinIdx = 0; vinIdx < vinNumSndCmd; vinIdx++ )
			vclSoundCmd[ vinIdx ].fvoPrecacheSndFiles( );

		vboAllPrecached = true;

	} // tclSoundCmdList::fvoPrecacheSndFiles

	void tclSoundCmdList::fvoListSounds( edict_t* pst_ptrEdict )
	{
		string vtxMsg;

		if ( !vboSoundsAreOn )
		{
			vtxMsg = "\n" + ctxMsgPrefix + "The sounds are currently disabled.";
			vclDebugTrace.fvoWriteEventInfo( true, vtxMsg );
			fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
			return;
		}

		vtxMsg = "\n" + ctxMsgPrefix + "Sound Command List ...";
		vclDebugTrace.fvoWriteEventInfo( true, vtxMsg );
		fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );

		for ( int vinIdx = cinMinSoundCmdIdx; 
					 vinIdx < vinNumSndCmd; 
					 vinIdx++ 
			 )
		{
			if ( !vclSoundCmd[ vinIdx ].vboEnabled )
			// do not list disabled sound commands;
				continue;

			vtxMsg = "  " + vclSoundCmd[ vinIdx ].ftxGetSndCmd( );
			vclDebugTrace.fvoWriteEventInfo( true, vtxMsg );
			fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
		}

		vtxMsg = ctxMsgPrefix + "End of List. Press <pgup>/<pgdn> to view.";
		vclDebugTrace.fvoWriteEventInfo( true, vtxMsg );
		fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );

	} // tclSoundCmdList::fvoListSounds

	bool tclSoundCmdList::fboSoundsAreOn( void )
	{
		return vboSoundsAreOn;
	} // tclSoundCmdList::fboSoundsAreOn

	bool tclSoundCmdList::fboSoundsAreOff( void )
	{
		return !vboSoundsAreOn;
	} // tclSoundCmdList::fboSoundsAreOff

	void tclSoundCmdList::fvoSetSoundsAreOn( bool pboSwitch )
	{
		if ( vboSoundsAreEnabled == false )
		{	
			vboSoundsAreOn = false;
			return;
		}
		vboSoundsAreOn = pboSwitch;
	} // tclSoundCmdList::fvoSetSoundsAreOn

	bool tclSoundCmdList::fboSoundsAreEnabled( void )
	{
		return vboSoundsAreEnabled;
	} // tclSoundCmdList::fboSoundsAreEnabled

	bool tclSoundCmdList::fboSoundsAreDisabled( void )
	{
		return !vboSoundsAreEnabled;
	} // tclSoundCmdList::fboSoundsAreDisabled

	void tclSoundCmdList::fvoSetSoundsAreEnabled( bool pboSwitch )
	{
		vboSoundsAreEnabled = pboSwitch;
		if ( fboSoundsAreDisabled( ) )
			vboSoundsAreOn = false;

	} // tclSoundCmdList::fvoSetSoundsAreEnabled

	void tclSoundCmdList::fvoSetSoundCmdSwitch( bool pboSwitch, string ptxSoundCmd )
	{
		int vinCmdIdx = finFindSoundCmd( ptxSoundCmd );
		if ( vinCmdIdx < cinMinSoundCmdIdx )
			return;
		fvoSetSoundCmdSwitch( pboSwitch, vinCmdIdx );
	} // tclSoundCmdList::fvoSetSoundCmdSwitch

	void tclSoundCmdList::fvoSetSoundCmdSwitch( bool pboSwitch, int pinCmdIdx )
	{
		if ( !finCmdIdxIsValid( pinCmdIdx ) )
		{ // Something is wrong with counter or index out of range.
			return;
		}

		vclSoundCmd[ pinCmdIdx ].vboEnabled = pboSwitch;

	} // tclSoundCmdList::fvoSetSoundCmdSwitch

	void tclSoundCmdList::fvoTurnSoundCmdOn( string ptxSoundCmd )
	{
		fvoSetSoundCmdSwitch( true, ptxSoundCmd );
	} // tclSoundCmdList::fvoTurnSoundCmdOn

	void tclSoundCmdList::fvoTurnSoundCmdOff( string ptxSoundCmd )
	{
		fvoSetSoundCmdSwitch( false, ptxSoundCmd );
	} // tclSoundCmdList::fvoTurnSoundCmdOff

	void tclSoundCmdList::fvoTurnSoundCmdOn( int pinCmdIdx )
	{
		fvoSetSoundCmdSwitch( true, pinCmdIdx );
	} // tclSoundCmdList::fvoTurnSoundCmdOn

	void tclSoundCmdList::fvoTurnSoundCmdOff( int pinCmdIdx )
	{
		fvoSetSoundCmdSwitch( false, pinCmdIdx );
	} // tclSoundCmdList::fvoTurnSoundCmdOff

	bool tclSoundCmdList::fboSoundCmdIsOn( string ptxSoundCmd )
	{
		int vinCmdIdx = finFindSoundCmd( ptxSoundCmd );
		if ( vinCmdIdx < 0 )
			return false;

		return vclSoundCmd[ vinCmdIdx ].vboEnabled;

	} // tclSoundCmdList::fboSoundCmdIsOn

	bool tclSoundCmdList::fboSoundCmdIsOn( int pinCmdIdx )
	{
		if ( !finCmdIdxIsValid( pinCmdIdx ) )
		{ // Something is wrong with counter or index out of range.
			return false;
		}

		return vclSoundCmd[ pinCmdIdx ].vboEnabled;

	} // tclSoundCmdList::fboSoundCmdIsOn

	const int cinUnknown = 0;
	const int cinSystem  = 1;
	const int cinAdmin   = 2;
	const int cinBot     = 3;
	const int cinDamage  = 4;
	const int cinPublic  = 5;

	const int cinExpecting_ListHeader				= 0;
	const int cinExpecting_SoundCmd					= 1;
	const int cinExpecting_SoundCmdOrListHeader  = 2;
	const int cinExpecting_SoundFile					= 3;
	const int cinExpecting_SoundFileOrOther		= 4;

	tclSoundCmdLoader vclSoundCmdLoader;

	tclSoundCmdLoader::tclSoundCmdLoader( )
	{
		vclSoundCmdList_Public.fvoSetSoundsAreEnabled( false );
		vclSoundCmdList_BotRandom.fvoSetSoundsAreEnabled( false );
		vclSoundCmdList_Damage.fvoSetSoundsAreEnabled( false );
	}; // tclSoundCmdLoader::tclSoundCmdLoader

	bool tclSoundCmdLoader::fboIsSoundListHeader( string ptxSoundLine )
	{

		if ( ptxSoundLine.length( ) < 2 )
			return false;

		if ( ptxSoundLine[ 0 ] != '[' )
			return false;

		if ( ptxSoundLine[ ptxSoundLine.length( ) - 1 ] != ']' )
			return false;

		return true;

	} // tclSoundCmdLoader::fboIsSoundListHeader

	bool tclSoundCmdLoader::fboIsSoundCommand( string ptxSoundLine )
	{
		string vtxMsg;

		if ( ptxSoundLine.length( ) < 1 )
			return false;

		if ( ptxSoundLine[ 0 ] != '!' )
			return false;

		return true;

	} // tclSoundCmdLoader::fboIsSoundCommand

	bool tclSoundCmdLoader::fboIsSoundFilePath( string ptxSoundLine )
	{

		if ( ptxSoundLine.length( ) < 4 )
			return false;

		ptxSoundLine = ftxToLower( ptxSoundLine );

		if ( ptxSoundLine.substr( ptxSoundLine.length( ) - 4, 4 ) != ".wav" )
			return false;

		return true;

	} // tclSoundCmdLoader::fboIsSoundFilePath

	int tclSoundCmdLoader::finReturnSoundListHeaderId( string ptxSoundLine )
	{
		if ( !fboIsSoundListHeader( ptxSoundLine ) )
			return cinUnknown;

		if ( ptxSoundLine == ctxSound_System )
			return cinSystem;

		if ( ptxSoundLine == ctxSound_Admin )
			return cinAdmin;

		if ( ptxSoundLine == ctxSound_Damage )
			return cinDamage;

		if ( ptxSoundLine == ctxSound_BotRandom )
			return cinBot;
		
		if ( ptxSoundLine == ctxSound_Public )
			return cinPublic;

		return cinUnknown;

	} // tclSoundCmdLoader::finReturnSoundListHeaderId

	string tclSoundCmdLoader::ftxReturnValidSoundCommand( string ptxSoundLine )
	{
		string vtxMsg;
		if ( !fboIsSoundCommand( ptxSoundLine ) )
			return "";
	
		if ( ptxSoundLine.length( ) < 2 )
			return "";
	
		if ( ptxSoundLine.find( " ", 0 ) != string::npos ) 
			return "";
	
		if ( ptxSoundLine.find( "\t", 0 ) != string::npos ) 
			return "";
	
		ptxSoundLine.erase( 0, 1 );
		return ptxSoundLine;

	} // tclSoundCmdLoader::ftxReturnValidSoundCommand

	string tclSoundCmdLoader::ftxReturnValidSoundFilePath( string ptxSoundLine )
	{
		if ( !fboIsSoundFilePath( ptxSoundLine ) )
			return "";
	
		string vtxTestPath = ctxGameSubFolder + "\\" + "sound" + "\\" + ptxSoundLine;

		if ( !fboFileCanBeRead( vtxTestPath ) ) 
			return "";

		return ptxSoundLine;

	} // tclSoundCmdLoader::ftxReturnValidSoundFilePath

	bool tclSoundCmdLoader::fboAddSoundCommand( int pinCommandList, string ptxSoundCommand )
	{
		if ( pinCommandList == cinSystem )
			return vclSoundCmdList_System.finAddSoundCmd( ptxSoundCommand ) > -1;
		if ( pinCommandList == cinAdmin )
			return vclSoundCmdList_Admin.finAddSoundCmd( ptxSoundCommand ) > -1;
		if ( pinCommandList == cinBot )
			return vclSoundCmdList_BotRandom.finAddSoundCmd( ptxSoundCommand ) > -1;
		if ( pinCommandList == cinDamage )
			return vclSoundCmdList_Damage.finAddSoundCmd( ptxSoundCommand ) > -1;
		if ( pinCommandList == cinPublic )
			return vclSoundCmdList_Public.finAddSoundCmd( ptxSoundCommand ) > -1;

		return false;

	} // tclSoundCmdLoader::fboAddSoundCommand

	bool tclSoundCmdLoader::fboAddSoundFile( int pinCommandList, string ptxSoundFile )
	{
		if ( pinCommandList == cinSystem )
			return vclSoundCmdList_System.fboAddSound( ptxSoundFile ) > -1;
		if ( pinCommandList == cinAdmin )
			return vclSoundCmdList_Admin.fboAddSound( ptxSoundFile ) > -1;
		if ( pinCommandList == cinBot )
			return vclSoundCmdList_BotRandom.fboAddSound( ptxSoundFile ) > -1;
		if ( pinCommandList == cinDamage )
			return vclSoundCmdList_Damage.fboAddSound( ptxSoundFile ) > -1;
		if ( pinCommandList == cinPublic )
			return vclSoundCmdList_Public.fboAddSound( ptxSoundFile ) > -1;

		return false;

	} // tclSoundCmdLoader::fboAddSoundFile

	void tclSoundCmdLoader::fvoLoadSoundCommands( edict_t* pst_ptrEdict )
	{
		string vtxMsg;
		string vtxFileName = ctxGameSubFolder + "\\" + ctxMEANFolder + "\\sounds.cfg";

		vtxMsg  = ctxMsgPrefix + "Loading sound commands from file:\n  " + vtxFileName;		
		fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
		vclDebugTrace.fvoWriteEventInfo( true, vtxMsg );

		if ( !( fboFileExists( vtxFileName ) 
			        && 
			     fboFileCanBeRead( vtxFileName ) )  
			)
		{
			vtxMsg  = "** ERROR: cannot read from file\n";
			vtxMsg += ctxMsgPrefix + "Load was aborted due to an error."; 
			fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
			vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
			return;
		} 

		FILE* vna_ptrFile;
		char vch_aryLineItem[ 255 ];

		vna_ptrFile = fopen( vtxFileName.c_str( ), "r" );

		if ( vna_ptrFile == NULL )
		{
			vtxMsg  = "** ERROR: cannot open file";
			vtxMsg += ctxMsgPrefix + "Load was aborted due to an error."; 
			fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
			vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
			return;
		}

		int vinCmdCount = 0;
		int vinLineCount = 0;
		string vtxSoundLine;
		string vtxSoundFile;

		int vinCommandList = cinUnknown;
		string vtxSoundCommand;
		int vinExpecting = cinExpecting_ListHeader;

		while	( fgets( vch_aryLineItem, 255, vna_ptrFile ) != NULL ) 
		{
			vinLineCount++;
			vtxSoundLine = vch_aryLineItem;
			vtxSoundLine = ftxTrim( vtxSoundLine );

			if ( vtxSoundLine.empty( ) )
				continue;
			
			if ( vtxSoundLine[ 0 ] == ';' )
				continue;

			if ( vinExpecting == cinExpecting_ListHeader )
			{
				locProcess_CommandList:

				vinCommandList = finReturnSoundListHeaderId( vtxSoundLine );

				if ( vinCommandList == cinUnknown )
				{
					vtxMsg  = "** ERROR: Expecting a valid sound command header at line " + ftxIntToString( vinLineCount ) + "\n";
					vtxMsg += "**        Found this instead: " + vtxSoundLine;
					fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
					vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
					goto locAbort;
				}

				vinExpecting = cinExpecting_SoundCmdOrListHeader;
				continue;

			} // cinExpecting_ListHeader

			if ( vinExpecting == cinExpecting_SoundCmdOrListHeader )
			{

				if ( fboIsSoundCommand( vtxSoundLine ) )
				{ 
					vinExpecting == cinExpecting_SoundCmd;
					goto locProcess_SoundCommand;
				}

				if ( fboIsSoundListHeader( vtxSoundLine ) )
				{
					vinExpecting == cinExpecting_ListHeader;
					goto locProcess_CommandList;
				}

				vtxMsg  = "** WARNING: Expecting a valid sound command or header at line " + ftxIntToString( vinLineCount ) + "\n";
				vtxMsg += "**          Found this instead: " + vtxSoundLine;
				fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
				vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
				goto locAbort;

			} // cinExpecting_SoundCmdOrListHeader

			if ( vinExpecting == cinExpecting_SoundCmd )
			{
				locProcess_SoundCommand:

				vtxSoundCommand = ftxReturnValidSoundCommand( vtxSoundLine );

				if ( vtxSoundCommand.empty( ) )
				{
					vtxMsg  = "** ERROR: Expecting a valid sound command at line " + ftxIntToString( vinLineCount ) + "\n";
					vtxMsg += "**        Found this instead: " + vtxSoundLine;
					fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
					vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
					goto locAbort;
				}

   			if ( !fboAddSoundCommand( vinCommandList, vtxSoundCommand ) )
				{
					vtxMsg  = "** ERROR: Unable to add sound command at line " + ftxIntToString( vinLineCount ) + "\n";
					vtxMsg += "**        The command is: " + vtxSoundLine;
					fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
					vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
					goto locAbort;
				}

				vinCmdCount++;
				vinExpecting = cinExpecting_SoundFile;
				continue;

			} // cinExpecting_SoundCmd

			if ( vinExpecting == cinExpecting_SoundFile )
			{
				locProcess_SoundFile:

				vtxSoundFile = ftxReturnValidSoundFilePath( vtxSoundLine );

				if ( vtxSoundFile.empty( ) )
				{
					vtxMsg  = "** WARNING: Expecting a valid sound file at line " + ftxIntToString( vinLineCount ) + "\n";
					vtxMsg += "**          Found this instead: " + vtxSoundLine;
					fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
					vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
					goto locSkipSoundFile;
				}

   			if ( !fboAddSoundFile( vinCommandList, vtxSoundFile ) )
				{
					vtxMsg  = "** ERROR: Unable to add sound file at line " + ftxIntToString( vinLineCount ) + "\n";
					vtxMsg += "**        The sound file is: " + vtxSoundLine;
					fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
					vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
					goto locAbort;
				}
				locSkipSoundFile:
				vinExpecting = cinExpecting_SoundFileOrOther;
				continue;

			} // cinExpecting_SoundFile

			if ( vinExpecting == cinExpecting_SoundFileOrOther )
			{

				if ( fboIsSoundCommand( vtxSoundLine ) )
				{ 
					vinExpecting == cinExpecting_SoundCmd;
					goto locProcess_SoundCommand;
				}

				if ( fboIsSoundListHeader( vtxSoundLine ) )
				{
					vinExpecting == cinExpecting_ListHeader;
					goto locProcess_CommandList;
				}

				if ( fboIsSoundFilePath( vtxSoundLine ) )
				{
					vinExpecting == cinExpecting_SoundFile;
					goto locProcess_SoundFile;
				}

				vtxMsg  = "** ERROR: Expecting a valid sound command, header or file at line " + ftxIntToString( vinLineCount ) + "\n";
				vtxMsg += "**        Found this instead: " + vtxSoundLine;
				fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
				vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
				goto locAbort;

			} // cinExpecting_SoundFileOrOther

			vtxMsg  = "** ERROR: Internal Error(1) at line " + ftxIntToString( vinLineCount ) + "\n";
			vtxMsg += "**        The line is: " + vtxSoundLine;
			fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
			vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
			goto locAbort;

//.begin for debugging only
//			vtxMsg  = "  Loaded: " + vtxSoundLine;
//			fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
//			vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);
//.end
		}
		
		locAbort:

		fclose( vna_ptrFile );

		vtxMsg  = ctxMsgPrefix + "Sound command load completed.";		
		fvoConsolePrint( pst_ptrEdict, vtxMsg + "\n" );
		vclDebugTrace.fvoWriteEventInfo( true, vtxMsg);

	} // tclSoundCmdLoader::fvoLoadSoundCommands

	void tclSoundCmdLoader::fvoLoadSoundCommands( )
	{
		fvoLoadSoundCommands( NULL );
	} // tclSoundCmdLoader::fvoLoadSoundCommands

// *** 
// *** tclSoundCmdList
// *** END.
